@@ -1,12 +1,12 @@
 ;;; desktop-environment.el --- Helps you control your GNU/Linux computer  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2018  Damien Cassou
+;; Copyright (C) 2018-2023  Damien Cassou
 
 ;; Author: Damien Cassou <damien@cassou.me>, Nicolas Petton <nicolas@petton.fr>
-;; Url: https://gitlab.petton.fr/DamienCassou/desktop-environment
-;; Package-requires: ((emacs "25.1"))
-;; Package-Version: 0.5.0
-;; Package-Revision: 9da8f4bddb78
+;; URL: https://gitlab.petton.fr/DamienCassou/desktop-environment
+;; Package-Requires: ((emacs "25.1"))
+;; Package-Version: 20250821.1428
+;; Package-Revision: 1f16fa0fcc1b
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -132,9 +132,21 @@ replaced by the desired new volume level."
   :type 'string)
 
 (defcustom desktop-environment-volume-toggle-command "amixer set Master toggle"
-  "Shell command toggling between muted and working."
+  "Shell command toggling between muted and working.
+If you change this variable, you might want to change
+`desktop-environment-volume-toggle-regexp' as well."
   :type 'string)
 
+(defcustom desktop-environment-volume-toggle-regexp (rx (or "[on]" "[off]"))
+  "Regular expression matching volume state (muted or working).
+
+This regular expression will be tested against the result of
+`desktop-environment-volume-toggle-command' and
+`desktop-environment-volume-toggle-microphone-command' and group
+0 must match the current volume state.  Set to nil if the toggle
+command generates no output."
+  :type 'regexp)
+
 (defcustom desktop-environment-volume-toggle-microphone-command "amixer set Capture toggle"
   "Shell command toggling microphone between muted and working."
   :type 'string)
@@ -143,14 +155,34 @@ replaced by the desired new volume level."
 ;;; Customization - screenshots
 
 (defcustom desktop-environment-screenshot-command "scrot"
-  "Shell command taking a screenshot in the current working directory."
+  "Shell command taking a screenshot in the current working directory.
+
+In order to support taking delayed screenshots, ensure that the
+argument specified in `desktop-environment-screenshot-delay-argument' is
+compatible with this command."
   :type 'string)
 
 (defcustom desktop-environment-screenshot-partial-command "scrot -s"
   "Shell command taking a partial screenshot in the current working directory.
 
 The shell command should let the user interactively select the
-portion of the screen."
+portion of the screen.
+
+In order to support taking delayed screenshots, ensure that the
+argument specified in `desktop-environment-screenshot-delay-argument'
+is compatible with this command."
+  :type 'string)
+
+(defcustom desktop-environment-screenshot-delay-argument "--delay %d"
+  "Argument to append to the screenshot shell command to delay the screenshot.
+
+When one of the screenshot
+functions (`desktop-environment-screenshot' or
+`desktop-environment-screenshot-part') is called with a positive
+integer as its first (prefix) argument, this string will be
+appended to either `desktop-environment-screenshot-command' or
+`desktop-environment-screenshot-partial-command' respectively,
+replacing the placeholder %d with the prefix argument."
   :type 'string)
 
 (defcustom desktop-environment-screenshot-directory "~/Pictures"
@@ -192,6 +224,10 @@ portion of the screen."
   "Shell command for going to next song."
   :type 'string)
 
+(defcustom desktop-environment-music-stop-command "playerctl stop"
+  "Shell command for stopping the music player instance."
+  :type 'string)
+
 
 ;;; Customization - EXWM keybindings
 
@@ -207,7 +243,7 @@ portion of the screen."
 (defun desktop-environment--shell-command-to-string (command)
   "Execute shell command COMMAND locally and return its output as a string."
    (let ((default-directory temporary-file-directory))
-      (shell-command-to-string command)))
+      (string-trim (shell-command-to-string command))))
 
 ;;; Helper functions - brightness
 
@@ -334,28 +370,44 @@ portion of the screen."
 (defun desktop-environment-toggle-mute ()
   "Toggle between muted and un-muted."
   (interactive)
-  (message "%s"
-           (desktop-environment--shell-command-to-string desktop-environment-volume-toggle-command)))
+  (message (let ((output (desktop-environment--shell-command-to-string desktop-environment-volume-toggle-command)))
+             (if desktop-environment-volume-toggle-regexp
+                 (format
+                  "Sound %s"
+                  (save-match-data
+                    (string-match desktop-environment-volume-toggle-regexp output)
+                    (match-string 0 output)))
+               "Sound toggled"))))
 
 ;;;###autoload
 (defun desktop-environment-toggle-microphone-mute ()
   "Toggle microphone between muted and un-muted."
   (interactive)
-  (message "%s"
-           (desktop-environment--shell-command-to-string desktop-environment-volume-toggle-microphone-command)))
+  (message (let ((output (desktop-environment--shell-command-to-string desktop-environment-volume-toggle-microphone-command)))
+             (if desktop-environment-volume-toggle-regexp
+                 (format
+                  "Mic %s"
+                  (save-match-data
+                    (string-match desktop-environment-volume-toggle-regexp output)
+                    (match-string 0 output)))
+               "Mic toggled"))))
 
 
 ;;; Commands - keyboard backlight
 ;;;###autoload
 (defun desktop-environment-keyboard-backlight-increment ()
-  "Increment keyboard backlight by `desktop-environment-keyboard-backlight-normal-increment'."
+  "Increment keyboard backlight.
+The increment step is configured with
+`desktop-environment-keyboard-backlight-normal-increment'."
   (interactive)
   (desktop-environment-keyboard-backlight-set
    (+ desktop-environment-keyboard-backlight-normal-increment
       (desktop-environment-keyboard-backlight-get))))
 
 (defun desktop-environment-keyboard-backlight-decrement ()
-  "Decrement keyboard backlight by `desktop-environment-keyboard-backlight-normal-decrement'."
+  "Decrement keyboard backlight.
+The decrement step is configured with
+`desktop-environment-keyboard-backlight-normal-decrement'."
   (interactive)
   (desktop-environment-keyboard-backlight-set
    (+ desktop-environment-keyboard-backlight-normal-decrement
@@ -365,22 +417,58 @@ portion of the screen."
 ;;; Commands - screenshots
 
 ;;;###autoload
-(defun desktop-environment-screenshot ()
-  "Take a screenshot of the screen in the current working directory."
-  (interactive)
-  (let ((default-directory (expand-file-name desktop-environment-screenshot-directory)))
-    (start-process-shell-command "desktop-environment-screenshot" nil desktop-environment-screenshot-command)))
+(defun desktop-environment-screenshot (&optional delay)
+  "Take a screenshot of the screen.
+
+Screenshots are stored in the directory
+`desktop-environment-screenshot-directory'.
+
+When DELAY is a positive integer, delay taking the screenshot by
+DELAY seconds.  When the function is called interactively, DELAY
+is the provided prefix argument.
+
+In order to delay the screenshot,
+`desktop-environment-screenshot-delay-argument' is appended to
+the command `desktop-environment-screenshot-command'."
+  (interactive "P")
+  (let ((default-directory (expand-file-name desktop-environment-screenshot-directory))
+        (command (if (and delay
+                          (numberp delay)
+                          (> delay 0))
+                     (concat desktop-environment-screenshot-command
+                             " "
+                             (format desktop-environment-screenshot-delay-argument delay))
+                   desktop-environment-screenshot-command)))
+    (start-process-shell-command "desktop-environment-screenshot" nil command)))
 
 ;;;###autoload
-(defun desktop-environment-screenshot-part ()
-  "Take a partial screenshot in the current working directory.
+(defun desktop-environment-screenshot-part (&optional delay)
+  "Take a screenshot of part of the screen.
+
+Screenshots are stored in the directory
+`desktop-environment-screenshot-directory'.
 
 The command asks the user to interactively select a portion of
-the screen."
-  (interactive)
-  (let ((default-directory (expand-file-name desktop-environment-screenshot-directory)))
+the screen.
+
+When DELAY is a positive integer, delay taking the screenshot by
+DELAY seconds.  When the function is called interactively, DELAY
+is the provided prefix argument.
+
+In order to delay the screenshot,
+`desktop-environment-screenshot-delay-argument' is appended to
+the command `desktop-environment-screenshot-partial-command'."
+  (interactive "P")
+  (let ((default-directory (expand-file-name desktop-environment-screenshot-directory))
+        (command (if (and delay
+                          (numberp delay)
+                          (> delay 0))
+                     (concat desktop-environment-screenshot-partial-command
+                             " "
+                             (format desktop-environment-screenshot-delay-argument delay))
+                   desktop-environment-screenshot-partial-command)))
     (message "Please select the part of your screen to shoot.")
-    (start-process-shell-command "desktop-environment-screenshot" nil desktop-environment-screenshot-partial-command)))
+    (start-process-shell-command "desktop-environment-screenshot" nil command)))
 
 
 ;;; Commands - screen locking
@@ -432,6 +520,12 @@ the screen."
   (message "%s"
            (desktop-environment--shell-command-to-string desktop-environment-music-next-command)))
 
+(defun desktop-environment-music-stop ()
+  "Stop music player instance."
+  (interactive)
+  (message "%s"
+           (desktop-environment--shell-command-to-string desktop-environment-music-stop-command)))
+
 ;;; Minor mode
 
 (defvar desktop-environment-mode-map
@@ -448,7 +542,7 @@ the screen."
            (,(kbd "S-<XF86AudioLowerVolume>") . ,(function desktop-environment-volume-decrement-slowly))
            (,(kbd "<XF86AudioMute>") . ,(function desktop-environment-toggle-mute))
            (,(kbd "<XF86AudioMicMute>") . ,(function desktop-environment-toggle-microphone-mute))
-           ;; Volume
+           ;; Screenshot
            (,(kbd "S-<print>") . ,(function desktop-environment-screenshot-part))
            (,(kbd "<print>") . ,(function desktop-environment-screenshot))
            ;; Screen locking
@@ -460,8 +554,10 @@ the screen."
            (,(kbd "<XF86Bluetooth>") . ,(function desktop-environment-toggle-bluetooth))
            ;; Music controls
            (,(kbd "<XF86AudioPlay>") . ,(function desktop-environment-toggle-music))
+           (,(kbd "<XF86AudioPause>") . ,(function desktop-environment-toggle-music))
            (,(kbd "<XF86AudioPrev>") . ,(function desktop-environment-music-previous))
-           (,(kbd "<XF86AudioNext>") . ,(function desktop-environment-music-next))))
+           (,(kbd "<XF86AudioNext>") . ,(function desktop-environment-music-next))
+           (,(kbd "<XF86AudioStop>") . ,(function desktop-environment-music-stop))))
         (map (make-sparse-keymap)))
     (dolist (keybinding desktop-environment--keybindings)
       (define-key map (car keybinding) (cdr keybinding)))
